using Sample.Domain;
using Sample.Validation;

namespace Sample.Presentation
{
    public class PersonPresenter
    {
        private IPersonView view;

        private IBusinessRuleSet<Person> rulesInContext = new BusinessRuleSet<Person>(Person.Rules.FirstName, Person.Rules.LastName, Person.Rules.Address, Person.Rules.Age);

        public PersonPresenter(IPersonView view)
        {
            this.view = view;
        }

        public void Submit()
        {
            IBusinessRuleSet<Person> broken = CreateFromView().Validate();
            if (broken.Count > 0)
            {
                view.DisplayErrors(broken.Messages);
            }
            else
            {
                view.DisplaySuccess();
            }
        }

        private Person CreateFromView()
        {
            int age = 0;
            int.TryParse(view.Age, out age);

            return new Person(view.FirstName, view.LastName, view.Address, age, Country.CANADA, Gender.MALE);
        }
    }
}