using System.Collections.Generic;

namespace Sample.Validation
{
    public class BusinessRuleSet<T> : IBusinessRuleSet<T>
    {
        private IList<IBusinessRule<T>> rules;

        public BusinessRuleSet(params IBusinessRule<T>[] rules) : this(new List<IBusinessRule<T>>(rules))
        {
        }

        public BusinessRuleSet(IList<IBusinessRule<T>> rules)
        {
            this.rules = rules;
        }

        public IBusinessRuleSet<T> BrokenBy(T item)
        {
            IList<IBusinessRule<T>> brokenRules = new List<IBusinessRule<T>>();

            foreach (IBusinessRule<T> rule in rules)
            {
                if (! rule.IsSatisfiedBy(item))
                {
                    brokenRules.Add(rule);
                }
            }
            return new BusinessRuleSet<T>(brokenRules);
        }

        public IList<string> Messages
        {
            get
            {
                return new List<IBusinessRule<T>>(rules).ConvertAll<string>(delegate(IBusinessRule<T> rule)
                                                                                {
                                                                                    return rule.Description;
                                                                                });
            }
        }


        public int Count
        {
            get { return rules.Count; }
        }

        public bool Contains(IBusinessRule<T> rule)
        {
            return rules.Contains(rule);
        }
    }
}