using NUnit.Framework;
using Rhino.Mocks;
using Sample.Validation;

namespace Sample.Domain.Test
{
    [TestFixture]
    public class PersonTest
    {
        private MockRepository mockery;

        [SetUp]
        public void Setup()
        {
            mockery = new MockRepository();
        }

        [TearDown]
        public void TearDown()
        {
            mockery.VerifyAll();
        }

        [Test]
        public void ShouldValidateUsingBusinessRuleSet()
        {            
            IBusinessRuleSet<Person> mockRuleSet = mockery.CreateMock<IBusinessRuleSet<Person>>();
            IBusinessRuleSet<Person> mockBrokenRules = mockery.CreateMock<IBusinessRuleSet<Person>>();

            Person person = new Person("", "", "", 0, null, null,mockRuleSet);
            
            Expect.Call(mockRuleSet.BrokenBy(person)).Return(mockBrokenRules);
            mockery.ReplayAll();
            
            IBusinessRuleSet<Person> brokenRules = person.Validate();                        
            Assert.AreEqual(mockBrokenRules,brokenRules);
        }

        [Test]
        public void ShouldValidateFirstNameUsingDefaultRules()
        {
            Person person = new Person("", "JP", "Address", 1, Country.CANADA, Gender.MALE);
            IBusinessRuleSet<Person> broken = person.Validate();
            Assert.AreEqual(1,broken.Count);
            Assert.IsTrue(broken.Contains(Person.Rules.FirstName));
        }
    }
}