using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;
using Rhino.Mocks;
using Sample.Validation;

namespace Sample.Domain.Test
{
    [TestFixture]
    public class BusinessRuleSetTest
    {
        private const string FIRST_NAME_MESSAGE = "First name is required";
        private const string LAST_NAME_MESSAGE = "Last name is required";
        private MockRepository mockery;

        [SetUp]
        public void Setup()
        {
            mockery = new MockRepository();
        }

        [TearDown]
        public void TearDown()
        {
            mockery.VerifyAll();
        }

        [Test]
        public void ShouldRetrieveAllRulesBrokenByItem()
        {
            IBusinessRule<Person> mockRule1 = mockery.CreateMock<IBusinessRule<Person>>();
            IBusinessRule<Person> mockRule2 = mockery.CreateMock<IBusinessRule<Person>>();
            IBusinessRule<Person> mockRule3 = mockery.CreateMock<IBusinessRule<Person>>();

            Person person = new Person("", "", "", 0, null, null);
            Expect.Call(mockRule1.IsSatisfiedBy(person)).Return(false);
            Expect.Call(mockRule2.IsSatisfiedBy(person)).Return(false);
            Expect.Call(mockRule3.IsSatisfiedBy(person)).Return(true);
            
            
            mockery.ReplayAll();
            
            IBusinessRuleSet<Person> ruleSet = new BusinessRuleSet<Person>(mockRule1, mockRule2, mockRule3);
            IBusinessRuleSet<Person> brokenRules = ruleSet.BrokenBy(person);
            Assert.AreEqual(2,brokenRules.Count);
        }

        [Test]
        public void ShouldRetrieveMessagesForAllRules()
        {
            IBusinessRule<Person> mockRule1 = mockery.CreateMock<IBusinessRule<Person>>();
            IBusinessRule<Person> mockRule2 = mockery.CreateMock<IBusinessRule<Person>>();

            Expect.Call(mockRule1.Description).Return(FIRST_NAME_MESSAGE);
            Expect.Call(mockRule2.Description).Return(LAST_NAME_MESSAGE);

            mockery.ReplayAll();
            
            IBusinessRuleSet<Person> rules = new BusinessRuleSet<Person>(mockRule1, mockRule2);
            IList<string> ruleMessages = rules.Messages;
            
            Assert.AreEqual(2,rules.Count);
            Assert.Contains(FIRST_NAME_MESSAGE,(IList)ruleMessages);
            Assert.Contains(LAST_NAME_MESSAGE,(IList)ruleMessages);
        }
    }
}