using NUnit.Framework;
using Rhino.Mocks;
using Sample.Validation;

namespace Sample.Domain.Test
{
    [TestFixture]
    public class BusinessRuleTest
    {
        private MockRepository mockery;

        [SetUp]
        public void Setup()
        {
            mockery = new MockRepository();
        }

        [TearDown]
        public void TearDown()
        {
            mockery.VerifyAll();
        }

        [Test]
        public void ShouldValidateUsingRule()
        {
            IBusinessRule<Person> firstNameRule = new BusinessRule<Person>(null,null,delegate(Person person)
                                                                                       {
                                                                                           return ! string.IsNullOrEmpty(person.FirstName);
                                                                                       });

            Person personWithInvalidFirstName = new Person("", "", "", 0, null, null);
            Person personWithValidFirstName = new Person("JP", "", "", 0, null, null);
            
            Assert.IsFalse(firstNameRule.IsSatisfiedBy(personWithInvalidFirstName));   
            Assert.IsTrue(firstNameRule.IsSatisfiedBy(personWithValidFirstName));
            
            
        }

        [Test]
        public void BusinessRulesForATypeWithTheSameNameShouldBeEqual()
        {
            IBusinessRule<Person> firstNameRule = new BusinessRule<Person>("FirstName",null,delegate(Person person)
                                                                               {
                                                                                   return false;
                                                                               });

            IBusinessRule<Person> firstNameRule2 = new BusinessRule<Person>("FirstName",null,delegate(Person person)
                                                                               {
                                                                                   return false;
                                                                               });
            
            Assert.IsTrue(firstNameRule.Equals(firstNameRule2));
        }
    }
}